# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 Akretion LDTA (<http://www.akretion.com>).
#    @author Raphaël Valyi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from tools.translate import _
from osv import fields, osv

from datetime import datetime
from dateutil.relativedelta import relativedelta
from lxml import etree


class resource_resource(osv.osv):
    _inherit = "resource.resource"

    _columns = {
        'product_id' : fields.many2one("product.product", "Product", help="Will be used to override the user related product for resource costing is filled"),
    }
    
    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=100):
        ids = self.search(cr, uid, [('product_id', 'ilike', name)])
        if ids:
            data = self.read(cr, uid, ids, ['name'])
            res = [(item['id'], item['name']) for item in data]
            return res
        else:
            res = super(resource_resource, self).name_search(cr, uid, name, args, operator, context, limit)
            return res

resource_resource()


class hr_employee(osv.osv):
    _inherit = "hr.employee"

    def create_user(self, cr, uid, name, context):
        if not self.pool.get('res.users').search(cr, uid, [('name', '=', name)]):
            return self.pool.get('res.users').create(cr, 1, {'name': name, 'login': name}, context)
        return False
    

    def create(self, cr, uid, vals, context=None):
        if not vals.get('user_id', False):
            vals.update({'user_id': self.create_user(cr, uid, vals.get('name', False), context)})
        return super(hr_employee, self).create(cr, uid, vals, context)
                

    def write(self, cr, uid, ids, vals, context=None):
        if not vals.get('user_id', False):
            for employee in self.browse(cr, uid, ids):
                if not employee.resource_id.user_id:
                    vals.update({'user_id': self.create_user(cr, uid, employee.resource_id.name, context)})
                    break
        if vals.get('active', 'undef') != 'undef':
            for employee in self.browse(cr, uid, ids):
                if employee.user_id:
                    resource_ids = self.pool.get('resource.resource').search(cr, uid, [('user_id', '=', employee.user_id.id), ('active', '=', not vals['active'])])
                    self.pool.get('resource.resource').write(cr, uid, resource_ids, {'active': vals['active']})
        return super(hr_employee, self).write(cr, uid, ids, vals, context)
       

hr_employee()


class project_phase(osv.osv):
    _inherit = "project.phase"

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
	"""Adds extra resource management fields if phase isn't a template"""
        res = super(project_phase, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar)
        print "PPPPPPPPPPPPPPP", context
        if view_type == 'form' and context and (context.get('project_state') not in ['draft', 'template']) and res['fields'].get('resource_ids', False):
            if res['fields']['resource_ids'].get('views', False):
                view_obj = etree.fromstring(res['fields']['resource_ids']['views']['tree']['arch'])
                node = view_obj.xpath("//button[@name='set_checked']")[0]
                node.set('invisible', 'False')
                node = view_obj.xpath("//field[@name='checked']")[0]
                node.set('invisible', 'False')
                node = view_obj.xpath("//field[@name='resource_id']")[0]
                node.set('domain', "[('product_id', '=', product_id)]")
                node.set('invisible', 'False')                
                node = view_obj.xpath("//field[@name='state']")[0]
                node.set('invisible', 'False')
                node = view_obj.xpath("//field[@name='amount_resource_ordered']")[0]
                node.set('invisible', 'False')
                node = view_obj.xpath("//field[@name='amount_resource_invoiced']")[0]
                node.set('invisible', 'False')
                node = view_obj.xpath("//field[@name='resource_cost']")[0]
                node.set('invisible', 'False')
                res['fields']['resource_ids']['views']['tree']['arch'] = etree.tostring(view_obj)

                view_obj = etree.fromstring(res['fields']['resource_ids']['views']['form']['arch'])
                res['fields']['resource_ids']['views']['form']['arch'] = etree.tostring(view_obj)
        return res
    
    def _amount_resource_ordered(self, cr, uid, ids, name, args, context=None):
        res = {}
        for phase in self.browse(cr, uid, ids):
            total = 0
            for resource_allocation in phase.resource_ids:
                total += resource_allocation.amount_resource_ordered
            res[phase.id] = total
        return res

    def _amount_resource_invoiced(self, cr, uid, ids, name, args, context=None):
        res = {}
        for phase in self.browse(cr, uid, ids):
            total = 0
            for resource_allocation in phase.resource_ids:
                total += resource_allocation.amount_resource_invoiced
            res[phase.id] = total
        return res
    
    def _amount_resource_invoiceable(self, cr, uid, ids, name, args, context=None):
        res = {}
        for phase in self.browse(cr, uid, ids):
            res[phase.id] = phase.amount_resource_ordered * phase.progress * 0.01 - phase.amount_resource_invoiced
        return res
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {}
        context['compute_planned_attendances'] = False
        if vals.get('progress', False):
            data = self.read(cr, uid, ids, ['duration', 'date_start', 'log'])[0]
            log = data['log']
            vals['log'] = (log and log or "") + ("date %s; progress %s%%; start_date %s, duration %s, user_id %s\n" % (datetime.now(), vals.get('progress'), vals.get('date_start', data['date_start']), vals.get('duration', data['duration']), uid,))
        context['indirect_write'] = True
        print "***********", vals
        res = super(project_phase, self).write(cr, uid, ids, vals, context)
        context['compute_planned_attendances'] = True
        if vals.get('date_start', False) or vals.get('duration', False) or vals.get('resource_ids', False):
            for phase in self.browse(cr, uid, ids):
                self.pool.get('project.resource.allocation').compute_planned_attendances(cr, uid, [allocation.id for allocation in phase.resource_ids], context)
        return res
        
    def set_open(self, cr, uid, ids, *args): # Check list to set open
        res = super(project_phase, self).set_open(cr, uid, ids)
        undef_ir_ids = self.pool.get('ir.model.data').search(cr, uid, [['name', '=', 'undef_resource']])
        undef_res_id = self.pool.get('ir.model.data').read(cr, uid, undef_ir_ids, ['res_id'])[0]['res_id']
        for phase in self.browse(cr, uid, ids):
            # Check that the resources are allocated and that the allocations are checked
            for resource in phase.resource_ids:
                if resource.resource_id.id == undef_res_id:
                    raise osv.except_osv(_('Warning!'), _('You have to allocate all resources'))
                if not resource.checked:
                    raise osv.except_osv(_('Warning!'), _('You have to check / approve all resources allocations'))

            # Check that the phase is ordered if there are resources
            if phase.resource_ids:
                order_line_ids = self.pool.get('sale.order.line').search(cr, uid, [('phase_id', '=', phase.id)])
                if order_line_ids:
                    valid_order = False
                    for order_line in self.pool.get("sale.order.line").browse(cr, uid, order_line_ids):
                        if order_line.order_id.state not in ('draft', 'cancel'):
                            valid_order = True
                            break
                    if not valid_order:
                        raise osv.except_osv(_('Warning!'), _('None of the related phase sale orders are in a valid state!'))
                else:
                    raise osv.except_osv(_('Warning!'), _('This phase hasn\'t been ordered yet'))
        return res
    
    _columns = {
        'amount_resource_ordered': fields.function(_amount_resource_ordered, method=True, type="float", string='Amount Ordered'),#TODO store=True?
        'amount_resource_invoiced': fields.function(_amount_resource_invoiced, method=True, type="float", string='Amount Invoiced'),#TODO store=True?
        'amount_resource_invoiceable': fields.function(_amount_resource_invoiceable, method=True, type="float", string='Resource Invoiceable'),#TODO store=True?
        'log': fields.text("Log"),
        'pricelist_id': fields.many2one('product.pricelist', 'Cost Picelist', domain=[('type', '=', 'purchase')]),
        'related_analytic_account_id': fields.related('project_id', 'analytic_account_id', type='many2one', relation='account.analytic.account', string='Analytic Account', readonly=True)
    }
    
project_phase()


class hr_analytic_timesheet(osv.osv):
    _inherit = 'hr.analytic.timesheet'
    
    def _get_duration(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for hr_analytic_timesheet in self.browse(cr, uid, ids):
            res[hr_analytic_timesheet.id] = 8 #FIXME anything better?
        return res
    
    def validate_presence(self, cr, uid, ids, context=None):
        for hr_analytic_timesheet in self.browse(cr, uid, ids):
            if hr_analytic_timesheet.phase_id:
                self.write(cr, uid, [hr_analytic_timesheet.id], {'state': 'valid', 'journal_id': self.pool.get('account.analytic.journal').search(cr, uid, [('code', '=', 'TS')])[0]})
        return True
    
    def cancel_presence(self, cr, uid, ids, context=None):
        for hr_analytic_timesheet in self.browse(cr, uid, ids):
            if hr_analytic_timesheet.phase_id:
                self.write(cr, uid, [hr_analytic_timesheet.id], {'state': 'draft', 'journal_id': self.pool.get('account.analytic.journal').search(cr, uid, [('code', 'ilike', 'TS - PLA')])[0]})
        return True
    
    def create_leave(self, cr, uid, ids, context=None):
        for hr_analytic_timesheet in self.browse(cr, uid, ids):
            if hr_analytic_timesheet.phase_id:
                self.write(cr, uid, [hr_analytic_timesheet.id], {'state': 'canceled', 'journal_id': self.pool.get('account.analytic.journal').search(cr, uid, [('code', 'ilike', 'TS - MIS')])[0]})
        return True
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context = {} 
        for hr_obj in self.browse(cr, uid, ids):
            if vals.get('date', False):
                if hr_obj.phase_id.date_start and hr_obj.phase_id.duration:
                    format = len(vals['date']) > 12 and '%Y-%m-%d %H:%M:%S' or '%Y-%m-%d'
                    date, start_date, duration = datetime.strptime(vals['date'], format), datetime.strptime(hr_obj.phase_id.date_start, '%Y-%m-%d'), hr_obj.phase_id.duration -1
                    if float(int(duration)) != duration: # if it is a a non round number
                        duration+=1
                    i = 0
                    while i < duration:
                        if (start_date + relativedelta(days=i)).isoweekday() == 6: # add two days to duration per weekend
                            duration += 2
                            i += 2
                        elif (start_date + relativedelta(days=i)).isoweekday() == 7: # add one if start date is sunday
                            duration += 1
                            i += 1
                        else:
                            i +=1
                    if date > start_date + relativedelta(days=duration) or date < start_date: # check the interval 
                        raise osv.except_osv(_('Warning!'), _('The allocation should be in the time interval of the project\'s phase'))
                else:
                    raise osv.except_osv(_('Warning!'), _('You allocated resources for a phase who has no start date or duration'))
            res_id = self.pool.get('resource.resource').search(cr, uid, [('user_id', '=', hr_obj.user_id.id)])
            alloc_id = self.pool.get('project.resource.allocation').search(cr, uid, [('resource_id', 'in', res_id), ('phase_id', '=', hr_obj.phase_id.id)])
            if len(alloc_id) != 1:
                raise osv.except_osv(_('Warning!'), _('A same resource is allocated more than once'))
            self.pool.get('project.resource.allocation').write(cr, uid, alloc_id, {'checked': not context.get('force', False)}, context)
        res = super(hr_analytic_timesheet, self).write(cr, uid, ids, vals, context)
        return res

    _columns = {
        'duration':fields.function(_get_duration , method=True, type='date', string='Duration'),
        'state': fields.selection([('draft', 'Draft'),('valid', 'Valid'),('canceled', 'Leave')], 'State', select=True),
        'f____you_openerp_user_name': fields.related('user_id', 'name', type='char', size=64, string='User', readonly=True) #workaround so that OpenERP client doesn't clash with logged in user_id. Sighh
    }
    
    _defaults = {
        'state': 'draft',
    }

hr_analytic_timesheet()


class project_resource_allocation(osv.osv):
    _inherit = 'project.resource.allocation'
    
    def set_checked(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'checked': True})
        return True

    def product_id_change(self, cr, uid, ids, product_id, project_id):
        res = {}
        if product_id:
            model_data_class = self.pool.get('ir.model.data')
            model_data_ids = model_data_class.search(cr, uid, [['name', '=', 'undef_resource']])
            if model_data_ids:
                resource_id = model_data_class.read(cr, uid, model_data_ids, ['res_id'])[0]['res_id']
                res['value'] = {'resource_id': resource_id}
        return res
    
    def _get_attendance_ids(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):        
            user_id = resource_allocation.resource_id.user_id.id
            res[resource_allocation.id] = self.pool.get('hr.analytic.timesheet').search(cr, uid, [('user_id', '=', user_id), ('phase_id', '=', resource_allocation.phase_id.id)])
        return res
    
    def _get_planned_attendance_ids(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):        
            user_id = resource_allocation.resource_id.user_id.id
            res[resource_allocation.id] = self.pool.get('hr.analytic.timesheet').search(cr, uid, [('journal_id', 'ilike', 'TS - PLA'), ('user_id', '=', user_id), ('phase_id', '=', resource_allocation.phase_id.id)])
        return res

    def _get_cost(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            cost = 0 
            user_id = resource_allocation.resource_id.user_id.id
            if resource_allocation.resource_id.product_id:
                analytic_ids = self.pool.get('account.analytic.line').search(cr, uid, [('user_id', '=', user_id), ('product_id', '=', resource_allocation.resource_id.product_id.id), ('phase_id', '=', resource_allocation.phase_id.id)])
                for entry in self.pool.get('account.analytic.line').browse(cr, uid, analytic_ids):
                    cost += entry.amount
            res[resource_allocation.id] = - cost

        return res
    
    def _get_invoice_lines(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            if resource_allocation.sale_id:
                res[resource_allocation.id] = [inv_line.id for inv_line in resource_allocation.sale_id.invoice_lines]
            else:
                res[resource_allocation.id] = []
        return res
    
    def _amount_resource_ordered(self, cr, uid, ids, name, args, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            if resource_allocation.sale_id:
                res[resource_allocation.id] = resource_allocation.sale_id.price_subtotal
            else:
                res[resource_allocation.id] = 0
        return res

    def _amount_resource_invoiced(self, cr, uid, ids, name, args, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            total = 0
            for invoice_line in resource_allocation.invoice_line_ids:
                total += invoice_line.price_subtotal
            res[resource_allocation.id] = total
        return res

    def _net_amount_resource_ordered(self, cr, uid, ids, name, args, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            if resource_allocation.sale_id:
                res[resource_allocation.id] = resource_allocation.sale_id.price_unit * resource_allocation.sale_id.product_uom_qty
            else:
                res[resource_allocation.id] = 0
        return res

    def _net_amount_resource_invoiced(self, cr, uid, ids, name, args, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            total = 0
            for invoice_line in resource_allocation.invoice_line_ids:
                total += invoice_line.price_unit * invoice_line.quantity
            res[resource_allocation.id] = total
        return res

    def _net_amount_resource_invoiceable(self, cr, uid, ids, name, args, context=None):
        res = {}
        for resource_allocation in self.browse(cr, uid, ids):
            res[resource_allocation.id] = resource_allocation.net_amount_resource_ordered * resource_allocation.phase_id.progress * 0.01 - resource_allocation.net_amount_resource_invoiced
        return res
    
    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        res = super(project_resource_allocation, self).write(cr, uid, ids, vals, context)
        if (not context.get('indirect_write', False)) and vals.get('useability', False) or vals.get('resource_id', False):
            self.compute_planned_attendances(cr, uid, ids, context)
        return res
    
    def compute_planned_attendances(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        context.update({'force': True})
        if context.get('compute_planned_attendances', True):
            hr_obj = self.pool.get('hr.analytic.timesheet')

            alloc_user_ids = [alloc.user_id.id for alloc in self.browse(cr, uid, ids)]
            print "alloc_user_ids", alloc_user_ids

            for resource_allocation in self.browse(cr, uid, ids):
                if not resource_allocation.product_id or resource_allocation.product_id.resource_usage == 'indirect'\
                or resource_allocation.resource_id.user_id.name == "Undefined User": # That means the resource is a template or indirect TODO weak!
                    continue
                done_days = []

                orfan_attendances = hr_obj.search(cr, uid, [('phase_id', '=', resource_allocation.phase_id.id), ('user_id', 'not in', alloc_user_ids), ('state', '=', 'draft')])
                print "orfan_attendances", orfan_attendances
                hr_obj.unlink(cr, uid, orfan_attendances)

                if resource_allocation.attendance_ids:
                    planned_to_remove = []
                    for planned_attendance in resource_allocation.attendance_ids:
                        if planned_attendance.state != 'draft':
                            done_days.append(planned_attendance)
                        else:
                            planned_to_remove.append(planned_attendance)
                    hr_obj.unlink(cr, uid, [planned_attendance.id for planned_attendance in planned_to_remove])
                
                phase = resource_allocation.phase_id
                project = phase.project_id
                hr_obj = self.pool.get('hr.analytic.timesheet')
                product = resource_allocation.product_id

                vals = {
                        'name': phase.name,
                        'journal_id': self.pool.get('account.analytic.journal').search(cr, uid, [('code', 'ilike', 'TS - PLA')])[0],
                        'account_id': project.analytic_account_id.id,
                        'general_account_id': product.property_account_expense and product.property_account_expense.id or product.categ_id.property_account_expense_categ.id,
                        'unit_amount': 1,  #TODO sure?
                        'date': phase.date_start or datetime.now(),
                        'user_id': resource_allocation.resource_id.user_id.id,
                        'phase_id': phase.id,
                        'product_id': product.id,
                        }

                if phase.pricelist_id:
                    context['pricelist_id'] = phase.pricelist_id.id
                elif project.pricelist_id:
                    context['pricelist_id'] = project.pricelist_id.id
                vals.update(hr_obj.on_change_unit_amount(cr, uid, [], vals['product_id'], vals['unit_amount'], project.company_id.id, False, vals['journal_id'], context)['value'])
                context.update({'user_id': vals['user_id']})
                
                num_days = resource_allocation.load_duration - len(done_days)#FIXME a better way may be possible
                available = True
                i = 0
                pulled = 0
                filled =0
                while i < num_days + pulled + filled:
                    if i >= phase.duration + pulled:
                        available = False
                        break
                    date = (phase.date_start and datetime.strptime(phase.date_start, '%Y-%m-%d') or datetime.now()) + relativedelta(days=i)
                    if date.isoweekday() == 6: #jump Saturday
                        date = date + relativedelta(days=2)
                        i += 2
                        pulled += 2
                    elif date.isoweekday() == 7: #jump Sunday
                        date = date + relativedelta(days=1)
                        i += 1
                        pulled += 1
                    elif hr_obj.search(cr, uid, [('date', '=', date), ('user_id', '=', vals['user_id']), ('phase_id', '=', phase.id)]): #existing allocation on same phase
                        i += 1
                        filled += 1
                        continue
                    elif hr_obj.search(cr, uid, [('date', '=', date), ('user_id', '=', vals['user_id'])]): #allocation on other phase/project => allocate anyway but set as invalid
                         available = False
#                        i += 1
#                        filled += 1
#                        continue
                        
                    vals.update({'date': date})
                    hr_obj.create(cr, uid, vals.copy(), context)
                    i += 1
                if available:
                    self.write(cr, uid, resource_allocation.id, {'state': 'valid'}, context)
                else:
                    self.write(cr, uid, resource_allocation.id, {'state': 'invalid'}, context)


    _columns = {
        'product_id': fields.many2one('product.product', 'Resource Product', domain="[('type','=','service')]"),
        'sale_id' : fields.many2one("sale.order.line", "Order line"),
        'invoice_line_ids':fields.function(_get_invoice_lines , method=True, type='one2many', relation='account.invoice.line', string='Invoice Lines'),
        'amount_resource_ordered': fields.function(_amount_resource_ordered, method=True, type="float", string='Amount Ordered'),#TODO store=True?
        'amount_resource_invoiced': fields.function(_amount_resource_invoiced, method=True, type="float", string='Amount Invoiced'),#TODO store=True?
        'net_amount_resource_ordered': fields.function(_net_amount_resource_ordered, method=True, type="float", string='Amount Resource Ordered'),#TODO store=True?
        'net_amount_resource_invoiced': fields.function(_net_amount_resource_invoiced, method=True, type="float", string='Amount Resource Invoiced'),#TODO store=True?
        'net_amount_resource_invoiceable': fields.function(_net_amount_resource_invoiceable, method=True, type="float", string='Amount Resource Invoiceable'),#TODO store=True?
        'resource_cost': fields.function(_get_cost, method=True, type="float", string='Effective Cost'),#TODO store=True?
        'attendance_ids':fields.function(_get_attendance_ids , method=True, type='one2many', relation='hr.analytic.timesheet', string='Attendances'),
        'planned_attendance_ids':fields.function(_get_planned_attendance_ids , method=True, type='one2many', relation='hr.analytic.timesheet', string='Attendances'),
        'state': fields.selection((('draft', 'Draft'), ('assigned', 'Assigned'), ('valid', 'Valid'), ('invalid', 'Invalid')), 'State'),
        'checked': fields.boolean('Checked', help='means that automatic planned attendances have been validated'),
    }
    
    _defaults = {
        'state': 'draft'
    }

project_resource_allocation()


class account_analyic_account(osv.osv):
    _inherit = "account.analytic.account"

    _columns = {
        'pricelist_id': fields.many2one('product.pricelist', 'Cost Picelist', domain=[('type', '=', 'purchase')]),
    }

account_analyic_account()


class project_project(osv.osv):
    _inherit = "project.project"

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
	"""Adds extra phase management fields if phase isn't a template"""
        res = super(project_project, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar)
        if context and (context.get('project_state') not in ['draft', 'template']) and res['fields'].get('phase_ids', False):
            if res['fields']['phase_ids'].get('views', False):
                view_obj = etree.fromstring(res['fields']['phase_ids']['views']['tree']['arch'])
                child = view_obj.xpath("//field[@name='duration']")[0]
                field = etree.Element('field', attrib={'name':'state'})
                child.addnext(field)
                field = etree.Element('field', attrib={'name':'date_end'})
                child.addnext(field)
                field = etree.Element('field', attrib={'name':'progress'})
                child.addprevious(field)
                field = etree.Element('field', attrib={'name':'date_start'})
                child.addprevious(field)
                res['fields']['phase_ids']['views']['tree']['arch'] = etree.tostring(view_obj)
                res['fields']['phase_ids']['views']['tree']['fields'] ['progress'] = {'string': u'Progress', 'type': 'float'}
                res['fields']['phase_ids']['views']['tree']['fields'] ['date_start'] = {'string': u'Start Date', 'type': 'date'}
                res['fields']['phase_ids']['views']['tree']['fields'] ['date_end'] = {'string': u'End Date', 'type': 'date'}
                res['fields']['phase_ids']['views']['tree']['fields'] ['state'] = {'string': u'State', 'type': 'char'}
        return res
    
    def validate_project(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        res = super(project_project, self).validate_project(cr, uid, ids, context)
        if not context:
            context={}
        context['compute_planned_attendances'] = True
        for project in self.browse(cr, uid, ids):
            for phase in project.phase_ids:
                self.pool.get('project.resource.allocation').compute_planned_attendances(cr, uid, [resource_allocation.id for resource_allocation in phase.resource_ids], context)
        return True                      
    
    def get_order_lines(self, cr, uid, ids, context=None):
        order_lines = super(project_project, self).get_order_lines(cr, uid, ids, context)
        for project in self.browse(cr, uid, ids):
            for phase in project.phase_ids:
                for resource_allocation in phase.resource_ids:
                    if resource_allocation.product_id and not resource_allocation.sale_id:
                        order_lines.append({
                                       'line': {'phase_id': phase.id,
                                        'resource_allocation_id': resource_allocation.id,
                                        'product_id': resource_allocation.product_id.id,
                                        'product_uom_qty': resource_allocation.load_duration,#phase.duration * resource_allocation.useability * 0.01,i #TODO possibly convert!
                                        'product_uom': phase.product_uom.id, #TODO possibly use product UOM with conversion
                                        },
                                        'resource_model': 'project.resource.allocation',
                                        'resource_id': resource_allocation.id
                        })
        return order_lines
    
project_project()


class account_analytic_line(osv.osv):
    _inherit = 'account.analytic.line'

    _columns = {
        'phase_id': fields.many2one('project.phase', 'Phase'),
    }

    def on_change_unit_amount(self, cr, uid, id, prod_id, quantity, company_id,
            unit=False, journal_id=False, context=None):
        """Ugly override of original method to take a potential context pricelist into account. Yes the code is ugly: we just copied/pasted the original, no time to clean it. """
        if context==None:
            context={}
        if not journal_id:
            j_ids = self.pool.get('account.analytic.journal').search(cr, uid, [('type','=','purchase')])
            journal_id = j_ids and j_ids[0] or False
        if not journal_id or not prod_id:
            return {}
        product_obj = self.pool.get('product.product')
        analytic_journal_obj =self.pool.get('account.analytic.journal')
        product_price_type_obj = self.pool.get('product.price.type')
        j_id = analytic_journal_obj.browse(cr, uid, journal_id, context=context)
        prod = product_obj.browse(cr, uid, prod_id, context=context)
        result = 0.0

        if j_id.type <> 'sale':
            a = prod.product_tmpl_id.property_account_expense.id
            if not a:
                a = prod.categ_id.property_account_expense_categ.id
            if not a:
                raise osv.except_osv(_('Error !'),
                        _('There is no expense account defined ' \
                                'for this product: "%s" (id:%d)') % \
                                (prod.name, prod.id,))
        else:
            a = prod.product_tmpl_id.property_account_income.id
            if not a:
                a = prod.categ_id.property_account_income_categ.id
            if not a:
                raise osv.except_osv(_('Error !'),
                        _('There is no income account defined ' \
                                'for this product: "%s" (id:%d)') % \
                                (prod.name, prod_id,))
			
        flag = False
        # Compute based on pricetype
        product_price_type_ids = product_price_type_obj.search(cr, uid, [('field','=','standard_price')], context=context)
        pricetype = product_price_type_obj.browse(cr, uid, product_price_type_ids, context=context)[0]
        if journal_id:
            journal = analytic_journal_obj.browse(cr, uid, journal_id, context=context)
            if journal.type == 'sale':
                product_price_type_ids = product_price_type_obj.search(cr, uid, [('field','=','list_price')], context)
                if product_price_type_ids:
                    pricetype = product_price_type_obj.browse(cr, uid, product_price_type_ids, context=context)[0]
        # Take the company currency as the reference one
        if pricetype.field == 'list_price':
            flag = True
        ctx = context.copy()
        if unit:
            # price_get() will respect a 'uom' in its context, in order
            # to return a default price for those units
            ctx['uom'] = unit
        amount_unit = prod.price_get(pricetype.field, context=ctx)[prod.id]

        if context.get("pricelist_id", False):
            print "found pricelist!", context.get("pricelist_id", False)
            amount_unit = self.pool.get('product.pricelist').price_get(cr, uid, [context.get("pricelist_id", False)], prod_id, 1, context=context)[context.get("pricelist_id", False)] #Note: this is debatable if qty should be passed
            print "amount", amount_unit
        else:
            amount_unit = prod.price_get(pricetype.field, context=ctx)[prod.id]
        prec = self.pool.get('decimal.precision').precision_get(cr, uid, 'Account')
        amount = amount_unit * quantity or 1.0
        result = round(amount, prec)
        if not flag:
            result *= -1
        return {'value': {
            'amount': result,
            'general_account_id': a,
            }
        }

account_analytic_line()
